@@ -1,14 +1,15 @@
 package commands
 
 import (
-	"os"
 	"path/filepath"
 	"strings"
 
 	"github.com/git-lfs/git-lfs/errors"
 	"github.com/git-lfs/git-lfs/git"
 	"github.com/git-lfs/git-lfs/git/githistory"
+	"github.com/git-lfs/git-lfs/git/githistory/log"
 	"github.com/git-lfs/git-lfs/git/odb"
+	"github.com/git-lfs/git-lfs/localstorage"
 	"github.com/spf13/cobra"
 )
 
@@ -19,14 +20,21 @@ var (
 	// migrateExcludeRefs is a set of Git references to explicitly exclude
 	// in the migration.
 	migrateExcludeRefs []string
+
+	// migrateEverything indicates the presence of the --everything flag,
+	// and instructs 'git lfs migrate' to migrate all local references.
+	migrateEverything bool
+
+	// migrateVerbose enables verbose logging
+	migrateVerbose bool
 )
 
 // migrate takes the given command and arguments, *odb.ObjectDatabase, as well
 // as a BlobRewriteFn to apply, and performs a migration.
-func migrate(args []string, r *githistory.Rewriter, opts *githistory.RewriteOptions) {
+func migrate(args []string, r *githistory.Rewriter, l *log.Logger, opts *githistory.RewriteOptions) {
 	requireInRepo()
 
-	opts, err := rewriteOptions(args, opts)
+	opts, err := rewriteOptions(args, opts, l)
 	if err != nil {
 		ExitWithError(err)
 	}
@@ -60,8 +68,8 @@ func getObjectDatabase() (*odb.ObjectDatabase, error) {
 //
 // If any of the above could not be determined without error, that error will be
 // returned immediately.
-func rewriteOptions(args []string, opts *githistory.RewriteOptions) (*githistory.RewriteOptions, error) {
-	include, exclude, err := includeExcludeRefs(args)
+func rewriteOptions(args []string, opts *githistory.RewriteOptions, l *log.Logger) (*githistory.RewriteOptions, error) {
+	include, exclude, err := includeExcludeRefs(l, args)
 	if err != nil {
 		return nil, err
 	}
@@ -71,6 +79,7 @@ func rewriteOptions(args []string, opts *githistory.RewriteOptions) (*githistory
 		Exclude: exclude,
 
 		UpdateRefs: opts.UpdateRefs,
+		Verbose:    opts.Verbose,
 
 		BlobFn:         opts.BlobFn,
 		TreeCallbackFn: opts.TreeCallbackFn,
@@ -88,10 +97,10 @@ func rewriteOptions(args []string, opts *githistory.RewriteOptions) (*githistory
 //     arguments and the --include-ref= or --exclude-ref= flag(s) aren't given.
 //   - Include all references given in --include-ref=<ref>.
 //   - Exclude all references given in --exclude-ref=<ref>.
-func includeExcludeRefs(args []string) (include, exclude []string, err error) {
+func includeExcludeRefs(l *log.Logger, args []string) (include, exclude []string, err error) {
 	hardcore := len(migrateIncludeRefs) > 0 || len(migrateExcludeRefs) > 0
 
-	if len(args) == 0 && !hardcore {
+	if len(args) == 0 && !hardcore && !migrateEverything {
 		// If no branches were given explicitly AND neither
 		// --include-ref or --exclude-ref flags were given, then add the
 		// currently checked out reference.
@@ -102,6 +111,10 @@ func includeExcludeRefs(args []string) (include, exclude []string, err error) {
 		args = append(args, current.Name)
 	}
 
+	if migrateEverything && len(args) > 0 {
+		return nil, nil, errors.New("fatal: cannot use --everything with explicit reference arguments")
+	}
+
 	for _, name := range args {
 		// Then, loop through each branch given, resolve that reference,
 		// and include it.
@@ -114,17 +127,30 @@ func includeExcludeRefs(args []string) (include, exclude []string, err error) {
 	}
 
 	if hardcore {
+		if migrateEverything {
+			return nil, nil, errors.New("fatal: cannot use --everything with --include-ref or --exclude-ref")
+		}
+
 		// If either --include-ref=<ref> or --exclude-ref=<ref> were
 		// given, append those to the include and excluded reference
 		// set, respectively.
 		include = append(include, migrateIncludeRefs...)
 		exclude = append(exclude, migrateExcludeRefs...)
+	} else if migrateEverything {
+		localRefs, err := git.LocalRefs()
+		if err != nil {
+			return nil, nil, err
+		}
+
+		for _, ref := range localRefs {
+			include = append(include, ref.Name)
+		}
 	} else {
 		// Otherwise, if neither --include-ref=<ref> or
 		// --exclude-ref=<ref> were given, include no additional
 		// references, and exclude all remote references that are remote
 		// branches or remote tags.
-		remoteRefs, err := getRemoteRefs()
+		remoteRefs, err := getRemoteRefs(l)
 		if err != nil {
 			return nil, nil, err
 		}
@@ -138,7 +164,7 @@ func includeExcludeRefs(args []string) (include, exclude []string, err error) {
 // getRemoteRefs returns a fully qualified set of references belonging to all
 // remotes known by the currently checked-out repository, or an error if those
 // references could not be determined.
-func getRemoteRefs() ([]string, error) {
+func getRemoteRefs(l *log.Logger) ([]string, error) {
 	var refs []string
 
 	remotes, err := git.RemoteList()
@@ -146,6 +172,12 @@ func getRemoteRefs() ([]string, error) {
 		return nil, err
 	}
 
+	w := l.Waiter("migrate: Fetching remote refs")
+	if err := git.Fetch(remotes...); err != nil {
+		return nil, err
+	}
+	w.Complete()
+
 	for _, remote := range remotes {
 		refsForRemote, err := git.RemoteRefs(remote)
 		if err != nil {
@@ -197,41 +229,42 @@ func currentRefToMigrate() (*git.Ref, error) {
 
 // getHistoryRewriter returns a history rewriter that includes the filepath
 // filter given by the --include and --exclude arguments.
-func getHistoryRewriter(cmd *cobra.Command, db *odb.ObjectDatabase) *githistory.Rewriter {
+func getHistoryRewriter(cmd *cobra.Command, db *odb.ObjectDatabase, l *log.Logger) *githistory.Rewriter {
 	include, exclude := getIncludeExcludeArgs(cmd)
 	filter := buildFilepathFilter(cfg, include, exclude)
 
 	return githistory.NewRewriter(db,
-		githistory.WithFilter(filter), githistory.WithLogger(os.Stderr))
+		githistory.WithFilter(filter), githistory.WithLogger(l))
 }
 
 func init() {
 	info := NewCommand("info", migrateInfoCommand)
 	info.Flags().IntVar(&migrateInfoTopN, "top", 5, "--top=<n>")
-	info.Flags().StringVar(&migrateInfoAboveFmt, "above", "1mb", "--above=<n>")
+	info.Flags().StringVar(&migrateInfoAboveFmt, "above", "", "--above=<n>")
 	info.Flags().StringVar(&migrateInfoUnitFmt, "unit", "", "--unit=<unit>")
 
 	importCmd := NewCommand("import", migrateImportCommand)
+	importCmd.Flags().BoolVar(&migrateVerbose, "verbose", false, "Verbose logging")
 
 	RegisterCommand("migrate", nil, func(cmd *cobra.Command) {
-		// Adding flags directly to cmd.Flags() doesn't apply those
-		// flags to any subcommands of the root. Therefore, loop through
-		// each subcommand specifically, and include common arguments to
-		// each.
-		//
-		// Once done, link each orphaned command to the
-		// `git-lfs-migrate(1)` command as a subcommand (child).
+		cmd.PersistentFlags().StringVarP(&includeArg, "include", "I", "", "Include a list of paths")
+		cmd.PersistentFlags().StringVarP(&excludeArg, "exclude", "X", "", "Exclude a list of paths")
 
-		for _, subcommand := range []*cobra.Command{
-			importCmd, info,
-		} {
-			subcommand.Flags().StringVarP(&includeArg, "include", "I", "", "Include a list of paths")
-			subcommand.Flags().StringVarP(&excludeArg, "exclude", "X", "", "Exclude a list of paths")
+		cmd.PersistentFlags().StringSliceVar(&migrateIncludeRefs, "include-ref", nil, "An explicit list of refs to include")
+		cmd.PersistentFlags().StringSliceVar(&migrateExcludeRefs, "exclude-ref", nil, "An explicit list of refs to exclude")
+		cmd.PersistentFlags().BoolVar(&migrateEverything, "everything", false, "Migrate all local references")
 
-			subcommand.Flags().StringSliceVar(&migrateIncludeRefs, "include-ref", nil, "An explicit list of refs to include")
-			subcommand.Flags().StringSliceVar(&migrateExcludeRefs, "exclude-ref", nil, "An explicit list of refs to exclude")
-
-			cmd.AddCommand(subcommand)
+		cmd.PersistentPreRun = func(_ *cobra.Command, args []string) {
+			// Initialize local storage before running any child
+			// subcommands, since migrations require lfs.TempDir to
+			// be initialized within ".git/lfs/objects".
+			//
+			// When lfs.TempDir is initialized to "/tmp",
+			// hard-linking can fail when another filesystem is
+			// mounted at "/tmp" (such as tmpfs).
+			localstorage.InitStorageOrFail()
 		}
+
+		cmd.AddCommand(importCmd, info)
 	})
 }
