@@ -42,7 +42,9 @@ func trackCommand(cmd *cobra.Command, args []string) {
 		os.Exit(128)
 	}
 
-	lfs.InstallHooks(false)
+	if !cfg.Os.Bool("GIT_LFS_TRACK_NO_INSTALL_HOOKS", false) {
+		lfs.InstallHooks(false)
+	}
 
 	if len(args) == 0 {
 		listPatterns()
@@ -56,6 +58,7 @@ func trackCommand(cmd *cobra.Command, args []string) {
 	}
 
 	wd, _ := tools.Getwd()
+	wd = tools.ResolveSymlinks(wd)
 	relpath, err := filepath.Rel(config.LocalWorkingDir, wd)
 	if err != nil {
 		Exit("Current directory %q outside of git working directory %q.", wd, config.LocalWorkingDir)
@@ -94,7 +97,7 @@ ArgsLoop:
 			writeablePatterns = append(writeablePatterns, pattern)
 		}
 
-		Print("Tracking %q", pattern)
+		Print("Tracking %q", unescapeTrackPattern(encodedArg))
 	}
 
 	// Now read the whole local attributes file and iterate over the contents,
@@ -258,7 +261,7 @@ var (
 )
 
 func escapeTrackPattern(unescaped string) string {
-	var escaped string = unescaped
+	var escaped string = strings.Replace(unescaped, `\`, "/", -1)
 
 	for from, to := range trackEscapePatterns {
 		escaped = strings.Replace(escaped, from, to, -1)
