@@ -1,6 +1,9 @@
 package log
 
-import "fmt"
+import (
+	"fmt"
+	"time"
+)
 
 // ListTask is a Task implementation that logs all updates in a list where each
 // entry is line-delimited.
@@ -11,24 +14,30 @@ import "fmt"
 //   msg: ..., done
 type ListTask struct {
 	msg string
-	ch  chan string
+	ch  chan *Update
 }
 
 // NewListTask instantiates a new *ListTask instance with the given message.
 func NewListTask(msg string) *ListTask {
 	return &ListTask{
 		msg: msg,
-		ch:  make(chan string, 1),
+		ch:  make(chan *Update, 1),
 	}
 }
 
 // Entry logs a line-delimited task entry.
 func (l *ListTask) Entry(update string) {
-	l.ch <- fmt.Sprintf("%s\n", update)
+	l.ch <- &Update{
+		S:  fmt.Sprintf("%s\n", update),
+		At: time.Now(),
+	}
 }
 
 func (l *ListTask) Complete() {
-	l.ch <- fmt.Sprintf("%s: ...", l.msg)
+	l.ch <- &Update{
+		S:  fmt.Sprintf("%s: ...", l.msg),
+		At: time.Now(),
+	}
 	close(l.ch)
 }
 
@@ -38,4 +47,4 @@ func (l *ListTask) Throttled() bool { return false }
 
 // Updates implements the Task.Updates function and returns a channel of updates
 // to log to the sink.
-func (l *ListTask) Updates() <-chan string { return l.ch }
+func (l *ListTask) Updates() <-chan *Update { return l.ch }
