@@ -11,18 +11,19 @@ type NetrcFinder interface {
 	FindMachine(string) *netrc.Machine
 }
 
-func ParseNetrc(osEnv Env) (NetrcFinder, error) {
+func ParseNetrc(osEnv Env) (NetrcFinder, string, error) {
 	home, _ := osEnv.Get("HOME")
 	if len(home) == 0 {
-		return &noFinder{}, nil
+		return &noFinder{}, "", nil
 	}
 
 	nrcfilename := filepath.Join(home, netrcBasename)
 	if _, err := os.Stat(nrcfilename); err != nil {
-		return &noFinder{}, nil
+		return &noFinder{}, nrcfilename, nil
 	}
 
-	return netrc.ParseFile(nrcfilename)
+	f, err := netrc.ParseFile(nrcfilename)
+	return f, nrcfilename, err
 }
 
 type noFinder struct{}
