@@ -208,11 +208,11 @@ assert_attributes_count() {
   fi
 }
 
-assert_file_writable() {
+assert_file_writeable() {
   ls -l "$1" | grep -e "^-rw"
 }
 
-refute_file_writable() {
+refute_file_writeable() {
   ls -l "$1" | grep -e "^-r-"
 }
 
@@ -238,6 +238,14 @@ assert_hooks() {
   [ -x "$git_root/hooks/pre-push" ]
 }
 
+assert_clean_status() {
+  status="$(git status)"
+  echo "$status" | grep "working tree clean" || {
+    echo $status
+    git lfs status
+  }
+}
+
 # pointer returns a string Git LFS pointer file.
 #
 #   $ pointer abc-some-oid 123 <version>
@@ -324,6 +332,21 @@ clone_repo() {
   echo "$out"
 }
 
+# clone_repo_url clones a Git repository to the subdirectory $dir under $TRASHDIR.
+# setup_remote_repo() needs to be run first. Output is written to clone.log.
+clone_repo_url() {
+  cd "$TRASHDIR"
+
+  local repo="$1"
+  local dir="$2"
+  echo "clone git repository $repo to $dir"
+  out=$(git clone "$repo" "$dir" 2>&1)
+  cd "$dir"
+
+  git config credential.helper lfstest
+  echo "$out" > clone.log
+  echo "$out"
+}
 
 # clone_repo_ssl clones a repository from the test Git server to the subdirectory
 # $dir under $TRASHDIR, using the SSL endpoint.
@@ -381,10 +404,13 @@ clone_repo_clientcert() {
 setup_remote_repo_with_file() {
   local reponame="$1"
   local filename="$2"
+  local dirname="$(dirname "$filename")"
 
   setup_remote_repo "$reponame"
   clone_repo "$reponame" "clone_$reponame"
 
+  mkdir -p "$dirname"
+
   git lfs track "$filename"
   echo "$filename" > "$filename"
   git add .gitattributes $filename
